const clone = (obj) => Array.isArray(obj)
    ? obj.map((item) => clone(item))
    : obj instanceof Date
        ? new Date(obj.getTime())
        : obj && typeof obj === 'object'
            ? Object.getOwnPropertyNames(obj).reduce((o, prop) => {
                o[prop] = clone(obj[prop]);
                return o;
            }, {})
            : obj;

const dateSort = (a, b) => (!a && !b ? 0 : !a ? -1 : !b ? 1 : a.getTime() - b.getTime());

const deepEquals = (obj1, obj2) => obj1 === obj2 ||
    (Array.isArray(obj1)
        ? Array.isArray(obj2) &&
            obj1.length === obj2.length &&
            obj1.every((item, index) => deepEquals(item, obj2[index]))
        : obj1 instanceof Date
            ? obj2 instanceof Date && obj1.getDate() === obj2.getDate()
            : obj1 && typeof obj1 === 'object'
                ? obj2 &&
                    typeof obj2 === 'object' &&
                    Object.getOwnPropertyNames(obj1).length ===
                        Object.getOwnPropertyNames(obj2).length &&
                    Object.getOwnPropertyNames(obj1).every((prop) => deepEquals(obj1[prop], obj2[prop]))
                : obj1 === obj2);

function deepMerge(source, data) {
    if (typeof data !== 'object' ||
        !data ||
        !source ||
        Array.isArray(source) ||
        source instanceof Date) {
        return data;
    }
    const sourceProps = Object.getOwnPropertyNames(source);
    const dataProps = Object.getOwnPropertyNames(data);
    return dataProps
        .filter((prop) => !sourceProps.includes(prop))
        .reduce((result, prop) => {
        result[prop] = data[prop];
        return result;
    }, sourceProps.reduce((result, prop) => {
        result[prop] = dataProps.includes(prop)
            ? deepMerge(source[prop], data[prop])
            : source[prop];
        return result;
    }, {}));
}

const defaultSort = (a, b) => (!a && !b ? 0 : !a ? -1 : !b ? 1 : a < b ? -1 : a > b ? 1 : 0);

const flattenGroup = (group) => {
    var _a, _b;
    const items = (_a = group.items) !== null && _a !== void 0 ? _a : flattenGroups((_b = group.groups) !== null && _b !== void 0 ? _b : []);
    return items.reduce((results, item, index) => {
        const keyRowspan = index ? -1 : group.totalItems;
        const rows = {};
        Object.keys(group.key).forEach((key) => {
            rows[key] = keyRowspan;
        });
        Object.keys(item).forEach((key) => {
            rows[key] = 1;
        });
        results.push(Object.assign(Object.assign(Object.assign({}, group.key), item), { _rows: rows }));
        return results;
    }, []);
};
const flattenGroups = (groups) => groups.reduce((results, group) => results.concat(flattenGroup(group)), []);

function groupBy(array, keyOrKeysOrGrouping) {
    const keys = typeof keyOrKeysOrGrouping === 'string'
        ? [keyOrKeysOrGrouping]
        : Array.isArray(keyOrKeysOrGrouping)
            ? keyOrKeysOrGrouping
            : keyOrKeysOrGrouping.keys;
    const thenby = typeof keyOrKeysOrGrouping !== 'string' &&
        !Array.isArray(keyOrKeysOrGrouping)
        ? keyOrKeysOrGrouping.thenby
        : undefined;
    const groups = array.reduce((results, item) => {
        var _a;
        const group = results.find((g) => keys.every((key) => item[key] === g.key[key]));
        let data = {};
        let groupKey = {};
        Object.keys(item).forEach((key) => {
            if (!keys.some((k) => k === key)) {
                data[key] = item[key];
            }
            else if (!group) {
                groupKey[key] = item[key];
            }
        });
        if (group !== undefined) {
            (_a = group.items) === null || _a === void 0 ? void 0 : _a.push(data);
            group.totalItems++;
        }
        else {
            results.push({ key: groupKey, items: [data], totalItems: 1 });
        }
        return results;
    }, []);
    return thenby
        ? groups.map((group) => {
            var _a;
            return ({
                key: group.key,
                groups: groupBy((_a = group === null || group === void 0 ? void 0 : group.items) !== null && _a !== void 0 ? _a : [], thenby),
                totalItems: group.totalItems,
            });
        })
        : groups;
}

const sort = new Intl.Collator(undefined, {
    numeric: true,
    sensitivity: 'base',
}).compare;
const naturalSort = (a, b) => !a && !b ? 0 : !a ? -1 : !b ? 1 : sort(a, b);

const resolveProperty = (obj, property) => property
    ? property
        .split('.')
        .reduce((result, prop) => (result ? result[prop] : undefined), obj)
    : undefined;

var SortDirection;
(function (SortDirection) {
    SortDirection["ascending"] = "ascending";
    SortDirection["descending"] = "descending";
})(SortDirection || (SortDirection = {}));
const multipleSortFunction = (...props) => {
    return (a, b) => {
        let direction = 0;
        props.some((item) => {
            if (typeof item === 'string') {
                direction = naturalSort(resolveProperty(a, item), resolveProperty(b, item));
            }
            else {
                direction = item.compare
                    ? item.display
                        ? item.compare(item.display(a), item.display(b))
                        : item.property
                            ? item.compare(resolveProperty(a, item.property), resolveProperty(b, item.property))
                            : item.compare(a, b)
                    : item.display
                        ? naturalSort(item.display(a), item.display(b))
                        : naturalSort(resolveProperty(a, item.property), resolveProperty(b, item.property));
                if (direction && item.direction === SortDirection.descending) {
                    direction = -direction;
                }
            }
            return direction !== 0;
        });
        return direction;
    };
};

const numberSort = (a, b) => !a && !b ? 0 : !a ? -1 : !b ? 1 : a < b ? -1 : a > b ? 1 : 0;

const pageNums = (pageNum, totalPages, maxPages) => {
    if (totalPages <= maxPages) {
        return Array.from({ length: totalPages }, (_, i) => i + 1);
    }
    else {
        let startPage = pageNum - Math.floor(maxPages / 2);
        if (startPage < 1) {
            startPage = 1;
        }
        else if (startPage > totalPages - maxPages + 1) {
            startPage = totalPages - maxPages + 1;
        }
        return Array.from({ length: maxPages }, (_, i) => i + startPage);
    }
};

function randomString(length, from = '') {
    let result = '';
    const characters = from || 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789_';
    var charactersLength = characters.length;
    for (var i = 0; i < length; i++) {
        result += characters.charAt(Math.floor(Math.random() * charactersLength));
    }
    return result;
}

function trapFocus(element) {
    const KEYCODE_TAB = 9;
    element.addEventListener('keydown', function (e) {
        const isTabPressed = e.key === 'Tab' || e.keyCode === KEYCODE_TAB;
        if (!isTabPressed) {
            return;
        }
        const focusableEls = element.querySelectorAll('a[href]:not([disabled]), button:not([disabled]), textarea:not([disabled]), input:not([disabled]), select:not([disabled])');
        const firstFocusableEl = focusableEls[0];
        const lastFocusableEl = focusableEls[focusableEls.length - 1];
        if (e.shiftKey) {
            /* shift + tab */ if (document.activeElement === firstFocusableEl) {
                lastFocusableEl.focus();
                e.preventDefault();
            }
        } /* tab */
        else {
            if (document.activeElement === lastFocusableEl) {
                firstFocusableEl.focus();
                e.preventDefault();
            }
        }
    });
}

/*
 * Public API Surface of ez-functions
 */

/**
 * Generated bundle index. Do not edit.
 */

export { SortDirection, clone, dateSort, deepEquals, deepMerge, defaultSort, flattenGroup, flattenGroups, groupBy, multipleSortFunction, naturalSort, numberSort, pageNums, randomString, resolveProperty, trapFocus };
//# sourceMappingURL=ez-functions.mjs.map
